# -*- coding: utf-8 -*-
"""
/***************************************************************************
 NigtArchive
                                 A QGIS plugin
 NIGT Archive QGIS data
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2024-11-18
        git sha              : $Format:%H$
        copyright            : (C) 2024 by National Institute of Green Technology
        email                : test@nigt.re.kr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import urllib.request
from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication, Qt
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction, QListWidgetItem, QMessageBox, QDialogButtonBox, QLineEdit

from qgis.core import (
    QgsProject, 
    QgsVectorLayer
)

# Initialize Qt resources from file resources.py
from .resources import *
# Import the code for the dialog
from .nigt_archive_dialog import NigtArchiveDialog
from .archive_login_dialog import ArchiveLoginDialog
import os.path
import configparser as cfg

# mg-241118
import requests
import json
from pathlib import Path
from functools import reduce
import urllib

class NigtArchive:
    """QGIS Plugin Implementation."""

    props = cfg.ConfigParser()
    ARCHIVE_URL = "http://dev.openankus.org:64083/" # "https://ctis.re.kr/dataarchive/admin/"  
    CTIS_URL = "http://118.67.150.175:8080/ko/" # "https://ctis.re.kr/ko/"
    ARCHIVE_DIR = ""
    archive_data = []
    category = []
    userid = ""
    _at = ""
    _rt = ""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'NigtArchive_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&NIGT Archive')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None

        # mg-241216
        basedir = os.path.dirname(os.path.abspath(__file__))
        os.chdir(basedir)
        self.props.read('./archive_config.ini')

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('NigtArchive', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/nigt_archive/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'NIGT Archive'),
            callback=self.run,
            parent=self.iface.mainWindow())

        # will be set False in run()
        self.first_start = True


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&NIGT Archive'),
                action)
            self.iface.removeToolBarIcon(action)


    def run(self):
        """Run method that performs all the real work"""

        # Create the dialog with elements (after translation) and keep reference
        # Only create GUI ONCE in callback, so that it will only load when the plugin is started
        if self.first_start == True:
            self.first_start = False

            self.popup = ArchiveLoginDialog()
            loginbtn = self.popup.buttonBox.button(QDialogButtonBox.Apply)
            # login button
            loginbtn.setText("Login")
            # click login
            loginbtn.clicked.connect(self.login)
            # edit id
            self.popup.idEdit.textChanged.connect(self.editLoginInfo)
            # edit password
            self.popup.pwEdit.textChanged.connect(self.editLoginInfo)
            self.popup.pwEdit.setEchoMode(QLineEdit.Password)

            self.popup.failMsg.setStyleSheet('QLabel#failMsg {color:red}')

            self.dlg = NigtArchiveDialog()
            self.dlg.listWidget.setSpacing(5)
            self.dlg.listWidget.itemClicked.connect(self.onSelectItem)

            self.dlg.comboBox.currentIndexChanged.connect(self.onSelectCategory)

            self.ARCHIVE_DIR = os.path.join(Path.home(), "nigt-archive-qgis")
            os.makedirs(self.ARCHIVE_DIR, exist_ok=True)

            try:
                self.userid = self.props["ETC"]["id"]
            except KeyError:
                self.props.add_section("ETC")
                self.userid = ""

        # init 
        self.popup.buttonBox.button(QDialogButtonBox.Apply).setDisabled(True)
        self.popup.pwEdit.setText("")
        self.popup.idEdit.setText(self.userid)

        # no tokens
        if self._at == "" or self._rt == "":
            self.popup.show()
            loginRes = self.popup.exec_()
            # login cancel
            if loginRes == False:
                return
            
        # init
        self.dlg.listWidget.clear()
        self.dlg.comboBox.clear()
        self.dlg.qgisNm.setText('')
        self.dlg.qgisDc.setPlainText('')
        self.dlg.file.setPlainText('')

        NigtArchive.archive_data = []
        NigtArchive.category = []

        self.dlg.comboBox.addItem('카테고리 선택')
        # param = {"order[0][name]":"frstRegisterPnttm", "order[0][dir]":"desc", "search[openYn]":True, "start":0, "length":20}
        # response = requests.get('http://192.168.50.41:8080/archive/admin/qgis/list.json', headers={"Content-Type":"application/json"}, params=param)

        # show the dialog
        self.dlg.show()    
        # get qgis list
        if self.getDataList() == False:
            self.dlg.hide()
            self._at = ""
            self._rt = ""
            return
        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result:
            # Do something useful here - delete the line containing pass and
            # substitute with your code.
            # pass

            # headers = {"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.93 Safari/537.36"}
            # list item
            for index in range(self.dlg.listWidget.count()):
                # checked
                if self.dlg.listWidget.item(index).checkState() == Qt.Checked:  
                    self.downloadFile(self.dlg.listWidget.item(index).data(Qt.UserRole))

    # login
    def login(self):
        id = self.popup.idEdit.text()
        pw = self.popup.pwEdit.text()
        # login
        response = requests.post(NigtArchive.CTIS_URL + 'infLogin.do', 
                                headers={"Content-Type":"application/x-www-form-urlencoded; charset=utf-8"},
                                data={'userId':id, 'pwd':pw})     

        # login success
        if response.status_code == 200:
            # fail
            if response.json()["result"] == False:
                # QMessageBox.critical(self.popup, 'Archive Login Fail', data["msg"])
                self.popup.failMsg.setText(response.json()["msg"])
            # pass
            else:
                self.userid = id

                self.props.set("ETC", "id", id)
                with open('./archive_config.ini', 'w') as f:
                    self.props.write(f)

                jsondat = response.json()['data']
                userdat = {
                    "userId" : jsondat['id']
                    , "userNm" : jsondat['name']
                    , "userLv" : jsondat['authCd']
                    , "deviceCode" : "" }
                # token
                resp = requests.post(NigtArchive.ARCHIVE_URL + 'member/loginPrc.do', 
                                headers={"Content-Type":"application/json; charset=utf-8"},
                                data=json.dumps(userdat))

                # token success
                if resp.status_code == 200:
                    token = resp.json()['tokens']
                    self._at = token['_at']
                    self._rt = token['_rt']
                    self.popup.accept()
                # token error
                else:
                    self.popup.failMsg.setText(resp.reason)
        # login error
        else:
            # QMessageBox.critical(self.popup, 'Archive Login Error', response.reason)
            self.popup.failMsg.setText(response.reason)


    def editLoginInfo(self):
        self.popup.buttonBox.button(QDialogButtonBox.Apply).setEnabled(self.popup.idEdit.text() != "" and self.popup.pwEdit.text() != "")

    # data list
    def getDataList(self):
        headers = {'Content-Type':"application/json; charset=utf-8", 'Access-Token':self._at , 'Refresh-Token':self._rt}
        # get qgis list
        # response = requests.get(NigtArchive.ARCHIVE_URL + 'admin/qgis/plugin-data.json', headers=headers)   
        response = requests.get(NigtArchive.ARCHIVE_URL + 'qgis/plugin-data.do', headers=headers)     
        # success
        if response.status_code == 200:
            data = response.json()
            # response success
            if data['result']:
                NigtArchive.archive_data = data["data"]
                NigtArchive.category = data["category"]

                # category list
                for cat in NigtArchive.category:
                    self.dlg.comboBox.addItem(cat["ctgName"], cat["ctgIdx"])

                # qgis data list
                for qitem in NigtArchive.archive_data:
                    self.addListItem(qitem)
                
                return True
            # response fail
            else:
                QMessageBox.critical(self, 'Archive Data', data['message'])    
        # error
        else:
            QMessageBox.critical(self, 'Archive Data', response.reason)
        
        return False


    def onSelectItem(self, item):
        id = item.data(Qt.UserRole)
        # response = requests.get(f'{NigtArchive.ARCHIVE_URL}admin/qgis/data.json?idx={qdata["qgisNo"]}', 
        #                        headers={"Content-Type":"application/json; charset=utf-8"})
        # success
        # if response.status_code == 200:
        #    json = response.json()
        #    data = json["data"]["data"]
        #    file = json["data"]["file"]

        data = self.findArchiveData(id)  
        if data is not None:
            self.dlg.qgisNm.setText(data["name"])
            self.dlg.qgisDc.setPlainText(data["desc"])
            self.dlg.file.setPlainText('\n'.join(f["fileName"] for f in data["fileList"])) # reduce(lambda t,f: t+'\n'+f["fileRealNm"], file))
                                
    def onSelectCategory(self):        
        self.dlg.listWidget.clear()

        # selected category id
        catId = self.dlg.comboBox.currentData()
        # data list
        for qitem in NigtArchive.archive_data:
            if catId is None:
                self.addListItem(qitem)
            # check category
            elif qitem["catId"] == catId:
                self.addListItem(qitem)

    def addListItem(self, data):
        item = QListWidgetItem(data["name"])
        item.setData(Qt.UserRole, data["id"])
        item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
        item.setCheckState(Qt.Unchecked)
        self.dlg.listWidget.addItem(item)

    def findArchiveData(self, idx):
        for qitem in NigtArchive.archive_data:
            # check category
            if qitem["id"] == idx:
                return qitem
        return None

    # download file
    def downloadFile(self, idx):
        data = self.findArchiveData(idx)
        if data is not None:
            # file list            
            for fl in data["fileList"]:   
                response = urllib.request.urlopen(f'{NigtArchive.ARCHIVE_URL}qgis/plugin-download.do?idx={fl["atchmnflSno"]}')
                
                # 저장경로
                pathname = os.path.join(self.ARCHIVE_DIR, fl["fileName"])

                with open(pathname, "wb") as file:                                             
                    # response = requests.get(f'{NigtArchive.ARCHIVE_URL}admin/qgis/plugin-download.do?idx={fl["atchmnflSno"]}', headers=headers)
                    file.write(response.read())    # content)
                    file.close()
                
                    name, ext = os.path.splitext(fl["fileName"])
                # add layer
                    if ext == '.shp' or ext == '.zip':
                        vlayer = QgsVectorLayer(pathname, data["name"], "ogr")
                        QgsProject.instance().addMapLayer(vlayer)        